"""The class implementation for the `smart_button` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSmartButtonIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `smart_button` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M22,9v6c0,1.1-0.9,2-2,2h-1l0-2h1V9H4v6h6v2H4c-1.1,0-2-0.9-2-2V9c0-1.1,0.9-2,2-2h16C21.1,7,22,7.9,22,9z M14.5,19 l1.09-2.41L18,15.5l-2.41-1.09L14.5,12l-1.09,2.41L11,15.5l2.41,1.09L14.5,19z M17,14l0.62-1.38L19,12l-1.38-0.62L17,10l-0.62,1.38 L15,12l1.38,0.62L17,14z M14.5,19l1.09-2.41L18,15.5l-2.41-1.09L14.5,12l-1.09,2.41L11,15.5l2.41,1.09L14.5,19z M17,14l0.62-1.38 L19,12l-1.38-0.62L17,10l-0.62,1.38L15,12l1.38,0.62L17,14z"/></g></svg>'  # noqa



"""Class implementation for the SVG text's italic mix-in.
"""

from typing import Dict
from typing import Optional

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


class SvgTextItalicMixIn(
    VariableNameMixIn,
    RevertMixIn,
):
    _italic: bool = False

    @property
    @add_debug_info_setting(module_name=__name__)
    def italic(self) -> Boolean:
        """
        Get a boolean value indicating whether a text is italic style or not.

        Returns
        -------
        italic_ : Boolean
            A boolean value indicating whether a text is italic style or not.
        """
        from apysc._expression import expression_data_util
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="italic",
        )
        italic_: Boolean = Boolean(self._italic, variable_name_suffix=suffix)
        expression: str = (
            f"{italic_.variable_name} = "
            f'{self.variable_name}.font("style") === "italic";'
        )
        expression_data_util.append_js_expression(expression=expression)
        return italic_

    @italic.setter
    @arg_validation_decos.is_apysc_boolean(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def italic(self, value: Boolean) -> None:
        from apysc._expression import expression_data_util

        self._italic = value._value
        expression: str = (
            f"if ({value.variable_name}) {{"
            f'\n  {self.variable_name}.font("style", "italic");'
            "\n} else {"
            f'\n  {self.variable_name}.font("style", "normal");'
            "\n}"
        )
        expression_data_util.append_js_expression(expression=expression)

    _italic_snapshots: Optional[Dict[str, bool]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_italic_snapshots",
            value=self._italic,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._italic = self._get_snapshot_val_if_exists(
            current_value=self._italic,
            snapshot_dict=self._italic_snapshots,
            snapshot_name=snapshot_name,
        )



"""The class implementation for the outlined `add_shopping_cart` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAddShoppingCartOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `add_shopping_cart` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M11 9h2V6h3V4h-3V1h-2v3H8v2h3v3zm-4 9c-1.1 0-1.99.9-1.99 2S5.9 22 7 22s2-.9 2-2-.9-2-2-2zm10 0c-1.1 0-1.99.9-1.99 2s.89 2 1.99 2 2-.9 2-2-.9-2-2-2zm-8.9-5h7.45c.75 0 1.41-.41 1.75-1.03l3.86-7.01L19.42 4l-3.87 7H8.53L4.27 2H1v2h2l3.6 7.59-1.35 2.44C4.52 15.37 5.48 17 7 17h12v-2H7l1.1-2z"/></svg>'  # noqa



"""The mix-in class implementation for the `x_axis_label_position`.
"""

from typing_extensions import final

from apysc._chart.x_axis_label_position import XAxisLabelPosition
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class XAxisLabelPositionMixIn:
    _x_axis_label_position: XAxisLabelPosition

    @final
    @arg_validation_decos.is_x_axis_label_position(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_x_axis_label_position(
        self,
        *,
        x_axis_label_position: XAxisLabelPosition,
    ) -> None:
        """
        Set an initial x-axis label position setting.

        Parameters
        ----------
        x_axis_label_position : XAxisLabelPosition
            An x-axis label position setting.
        """
        self._x_axis_label_position = x_axis_label_position



"""The class implementation for the `policy` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPolicyIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `policy` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g/><g><path d="M21,5l-9-4L3,5v6c0,5.55,3.84,10.74,9,12c2.3-0.56,4.33-1.9,5.88-3.71l-3.12-3.12c-1.94,1.29-4.58,1.07-6.29-0.64 c-1.95-1.95-1.95-5.12,0-7.07c1.95-1.95,5.12-1.95,7.07,0c1.71,1.71,1.92,4.35,0.64,6.29l2.9,2.9C20.29,15.69,21,13.38,21,11V5z"/><circle cx="12" cy="12" r="3"/></g></g></svg>'  # noqa



"""The mix-in class implementation for the `SvgText`'s `_set_font_family` method.
"""

from typing import List
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._converter.list_of_strs_to_array import list_of_strs_to_array_of_string
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.string import String


class SvgTextSetFontFamilyMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_font_family(
        self,
        *,
        font_family: Optional[Union[Array[String], List[str]]],
    ) -> None:
        """
        Set a font-family value.

        Parameters
        ----------
        font_family : Optional[Array[String]]
            A font-family setting.
        """
        from apysc._display.svg_text_font_family_mixin import SvgTextFontFamilyMixIn
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if not isinstance(self, SvgTextFontFamilyMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextFontFamilyMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )
        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="font_family",
        )
        font_family_: Optional[Array[String]] = list_of_strs_to_array_of_string(
            optional_list_or_arr=font_family,
            variable_name_suffix=suffix,
        )

        if font_family_ is None:
            return
        self.font_family = font_family_



"""Interface class implementation for the `__repr__` method.
"""

from abc import ABC
from abc import abstractmethod


class ReprInterface(ABC):
    @abstractmethod
    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).
        """



"""The class implementation for the outlined `rowing` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRowingOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `rowing` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M8.5 14.5L4 19l1.5 1.5L9 17h2l-2.5-2.5zM15 1c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6 20.01L18 24l-2.99-3.01V19.5l-7.1-7.09c-.31.05-.61.07-.91.07v-2.16c1.66.03 3.61-.87 4.67-2.04l1.4-1.55c.19-.21.43-.38.69-.5.29-.14.62-.23.96-.23h.03C15.99 6.01 17 7.02 17 8.26v5.75c0 .84-.35 1.61-.92 2.16l-3.58-3.58v-2.27c-.63.52-1.43 1.02-2.29 1.39L16.5 18H18l3 3.01z"/></svg>'  # noqa



"""The bottom y geometry mix-in implementation for the `RectangleGeom`.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class RectangleGeomBottomYMixIn:
    _bottom_y: Number

    @property
    @add_debug_info_setting(module_name=__name__)
    def bottom_y(self) -> Number:
        """
        Get the rectangle bottom y coordinate.

        Returns
        -------
        bottom_y : Number
            The rectangle bottom y coordinate.
        """
        return self._bottom_y



"""The class implementation for the `fiber_new` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFiberNewIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `fiber_new` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/></g><g><g><g><path d="M20,4H4C2.89,4,2.01,4.89,2.01,6L2,18c0,1.11,0.89,2,2,2h16c1.11,0,2-0.89,2-2V6C22,4.89,21.11,4,20,4z M8.5,15H7.3 l-2.55-3.5V15H3.5V9h1.25l2.5,3.5V9H8.5V15z M13.5,10.26H11v1.12h2.5v1.26H11v1.11h2.5V15h-4V9h4V10.26z M20.5,14 c0,0.55-0.45,1-1,1h-4c-0.55,0-1-0.45-1-1V9h1.25v4.51h1.13V9.99h1.25v3.51h1.12V9h1.25V14z"/></g></g></g></svg>'  # noqa



"""The class implementation for the `rtt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRttIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `rtt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><title>ic_dialer_rtt_revised_24px</title><path d="M9.03 3l-1.11 7.07h2.62l.7-4.5h2.58L11.8 18.43H9.47L9.06 21h7.27l.4-2.57h-2.35l2-12.86h2.58l-.71 4.5h2.65L22 3H9.03zM8 5H4l-.31 2h4L8 5zm-.61 4h-4l-.31 2h4l.31-2zm.92 8h-6L2 19h6l.31-2zm.62-4h-6l-.31 2h6.01l.3-2z"/></svg>'  # noqa



"""The mix-in class implementation for the `to_fixed` method.
"""

from typing import TYPE_CHECKING
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._type.int import Int


class ToFixedMixIn:
    @final
    @arg_validation_decos.is_apysc_int_or_number(arg_position_index=0)
    @arg_validation_decos.num_is_between(
        arg_position_index=1, min_value=0, max_value=100
    )
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def to_fixed(
        self,
        *,
        digits: Union[int, "Int"],
        variable_name_suffix: str = "",
    ) -> String:
        """
        Convert value to fixed floating point string notation.

        Parameters
        ----------
        digits : int or Int
            A floating point digit number (0 to 100 value is acceptable).
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        result_str : String
            A converted string.

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage()
        >>> num: ap.Number = ap.Number(10.789)
        >>> fixed_float_str: ap.String = num.to_fixed(digits=2)
        >>> fixed_float_str
        String("10.79")

        >>> fixed_float_str = num.to_fixed(digits=5)
        >>> fixed_float_str
        String("10.78900")

        >>> fixed_float_str = num.to_fixed(digits=0)
        >>> fixed_float_str
        String("11")
        """
        from apysc._expression import expression_data_util
        from apysc._type.int import Int
        from apysc._type.number import Number
        from apysc._type.variable_name_mixin import VariableNameMixIn

        result_str: String = String("", variable_name_suffix=variable_name_suffix)
        if isinstance(digits, int):
            digits_: Int = Int(digits, variable_name_suffix=variable_name_suffix)
        else:
            digits_ = digits
        variable_name: str = ""
        if isinstance(self, VariableNameMixIn):
            variable_name = self.variable_name
        expression: str = (
            f"{result_str.variable_name} = "
            f"{variable_name}.toFixed({digits_.variable_name});"
        )
        expression_data_util.append_js_expression(expression=expression)

        value: float = 0.0
        if isinstance(self, (Int, Number)):
            value = self._value
        value = round(value, digits_._value)
        py_fixed_floating_point_value: str = f"{value:.{digits_._value}f}"
        result_str._value = py_fixed_floating_point_value
        return result_str



"""The class implementation for the outlined `phone` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPhoneOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `phone` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M6.54 5c.06.89.21 1.76.45 2.59l-1.2 1.2c-.41-1.2-.67-2.47-.76-3.79h1.51m9.86 12.02c.85.24 1.72.39 2.6.45v1.49c-1.32-.09-2.59-.35-3.8-.75l1.2-1.19M7.5 3H4c-.55 0-1 .45-1 1 0 9.39 7.61 17 17 17 .55 0 1-.45 1-1v-3.49c0-.55-.45-1-1-1-1.24 0-2.45-.2-3.57-.57-.1-.04-.21-.05-.31-.05-.26 0-.51.1-.71.29l-2.2 2.2c-2.83-1.45-5.15-3.76-6.59-6.59l2.2-2.2c.28-.28.36-.67.25-1.02C8.7 6.45 8.5 5.25 8.5 4c0-.55-.45-1-1-1z"/></svg>'  # noqa



"""The class implementation for the `dialer_sip` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialDialerSipIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `dialer_sip` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M17 3h-1v5h1V3zm-2 2h-2V4h2V3h-3v3h2v1h-2v1h3V5zm3-2v5h1V6h2V3h-3zm2 2h-1V4h1v1zm0 10.5c-1.25 0-2.45-.2-3.57-.57-.35-.11-.74-.03-1.01.24l-2.2 2.2c-2.83-1.44-5.15-3.75-6.59-6.59l2.2-2.21c.27-.26.35-.65.24-1C8.7 6.45 8.5 5.25 8.5 4c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1 0 9.39 7.61 17 17 17 .55 0 1-.45 1-1v-3.5c0-.55-.45-1-1-1z"/></svg>'  # noqa



"""The mix-in class implementation for the material design's x and y
coordinates attributes.
"""

from typing import Union

from apysc._type.number import Number


class MaterialXAndYAttributesMixIn:

    def _set_x_and_y_coordinates(
        self, *, x: Union[float, Number], y: Union[float, Number]
    ) -> None:
        """
        Set the x and y coordinates to an instance button.

        Parameters
        ----------
        x : Union[float, Number]
            X-coordinate.
        y : Union[float, Number]
            Y-coordinate.
        """
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        self.x = to_apysc_val_from_builtin.get_copied_number_from_builtin_val(
            float_or_num=x,
            variable_name_suffix=get_attr_or_variable_name_suffix(
                instance=self,
                value_identifier="x",
            ),
        )
        self.y = to_apysc_val_from_builtin.get_copied_number_from_builtin_val(
            float_or_num=y,
            variable_name_suffix=get_attr_or_variable_name_suffix(
                instance=self,
                value_identifier="y",
            ),
        )



"""This module is for the translation mapping data of the
following document:

Document file: line.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Line class": "# Line クラス",
    ##################################################
    "This page explains the `Line` class.": "このページでは`Line`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `Line` class creates a straight-line vector graphics object.": "`Line`クラスは直線のベクターグラフィックスを生成します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `Line` class constructor requires the `start_point` and `end_point` arguments.": "`Line`クラスのコンストラクタでは`start_point`や`end_point`の引数指定を必要とします。",  # noqa
    ##################################################
    "The constructor also accepts each style's argument, such as the `line_color`.": "コンストラクタは`line_color`などのスタイル設定の引数も受け付けます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="line_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="line_basic_usage/")\n```',  # noqa
    ##################################################
    "## Note of the draw_line or other interfaces": "## draw_line や他の各インターフェイスの特記事項",
    ##################################################
    "You can also create a line instance with the `draw_line` interface (or the other interfaces, such as the `draw_dotted_line`).": "`draw_line`や`draw_dotted_line`などの他のインターフェイスを使う形でも直線のインスタンスを生成することができます。",  # noqa
    ##################################################
    "Please see also:": "関連資料:",
    ##################################################
    "- [Graphics class draw_line interface](graphics_draw_line.md)": "- [Graphics クラスの draw_line (線の描画)のインターフェイス](jp_graphics_draw_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dotted_line interface](graphics_draw_dotted_line.md)": "- [Graphics クラスの draw_dotted_line (点線の描画)のインターフェイス](jp_graphics_draw_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dashed_line interface](graphics_draw_dashed_line.md)": "- [Graphics クラスの draw_dashed_line (破線の描画)のインターフェイス](jp_graphics_draw_dashed_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_round_dotted_line interface](graphics_draw_round_dotted_line.md)": "- [Graphics クラスの draw_round_dotted_line (点線(丸)の描画)のインターフェイス](jp_graphics_draw_round_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dash_dotted_line interface](graphics_draw_dash_dotted_line.md)": "- [Graphics クラスの draw_dash_dotted_line (一点鎖線の描画)のインターフェイス](jp_graphics_draw_dash_dotted_line.md)",  # noqa
    ##################################################
    "## x property interface example": "## x属性のインターフェイス例",
    ##################################################
    "The `x` property updates or gets the instance's x-coordinate:": "`x`属性ではX座標の値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\nline.x = ap.Number(100)\n\nap.save_overall_html(dest_dir_path="line_x/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\nline.x = ap.Number(100)\n\nap.save_overall_html(dest_dir_path="line_x/")\n```',  # noqa
    ##################################################
    "Notes: This attribute's value becomes the same as the arguments' minimum point value.": "特記事項: この属性の値は引数の座標の最小値と同値になります。",  # noqa
    ##################################################
    "## y property interface example": "## y属性のインターフェイス例",
    ##################################################
    "The `y` property updates or gets the instance's y-coordinate:": "`y`属性ではY座標の値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\nline.y = ap.Number(80)\n\nap.save_overall_html(dest_dir_path="line_y/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\nline.y = ap.Number(80)\n\nap.save_overall_html(dest_dir_path="line_y/")\n```',  # noqa
    ##################################################
    "Notes: This attribute's value becomes the same as the arguments' minimum point value.": "特記事項: この属性の値は引数の座標の最小値と同値になります。",  # noqa
    ##################################################
    "## line_color property interface example": "## line_color属性のインターフェイス例",
    ##################################################
    "The `line_color` property updates or gets the instance's line color:": "`line_color`属性では線の色の値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_thickness=5,\n)\nline.line_color = ap.Color("#f0a")\n\nap.save_overall_html(dest_dir_path="line_line_color/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_thickness=5,\n)\nline.line_color = ap.Color("#f0a")\n\nap.save_overall_html(dest_dir_path="line_line_color/")\n```',  # noqa
    ##################################################
    "## line_alpha property interface example": "## line_alpha属性のインターフェイス例",
    ##################################################
    "The `line_alpha` property updates or gets the instance's line alpha (opacity):": "`line_alpha`属性では線の透明度の値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\nline.line_alpha = ap.Number(0.3)\n\nap.save_overall_html(dest_dir_path="line_line_alpha/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\nline.line_alpha = ap.Number(0.3)\n\nap.save_overall_html(dest_dir_path="line_line_alpha/")\n```',  # noqa
    ##################################################
    "## line_thickness property interface example": "## line_thickness属性のインターフェイス例",
    ##################################################
    "The `line_thickness` property updates or gets the instance's line thickness (line width):": "`line_thickness`属性では線の幅の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n)\nline.line_thickness = ap.Int(10)\n\nap.save_overall_html(dest_dir_path="line_line_thickness/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n)\nline.line_thickness = ap.Int(10)\n\nap.save_overall_html(dest_dir_path="line_line_thickness/")\n```',  # noqa
    ##################################################
    "## line_dot_setting property interface example": "## line_dot_setting属性のインターフェイス例",  # noqa
    ##################################################
    "The `line_dot_setting` property updates or gets the instance's line dot-style setting:": "`line_dot_setting`属性では点線のスタイル設定の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nline.line_dot_setting = ap.LineDotSetting(dot_size=3)\n\nap.save_overall_html(dest_dir_path="line_line_dot_setting/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nline.line_dot_setting = ap.LineDotSetting(dot_size=3)\n\nap.save_overall_html(dest_dir_path="line_line_dot_setting/")\n```',  # noqa
    ##################################################
    "## line_dash_setting property interface example": "## line_dash_setting属性のインターフェイス例",  # noqa
    ##################################################
    "The `line_dash_setting` property updates or gets the instance's line dash-style setting:": "`line_dash_setting`属性では破線のスタイル設定の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nline.line_dash_setting = ap.LineDashSetting(dash_size=6, space_size=2)\n\nap.save_overall_html(dest_dir_path="line_line_dash_setting/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nline.line_dash_setting = ap.LineDashSetting(dash_size=6, space_size=2)\n\nap.save_overall_html(dest_dir_path="line_line_dash_setting/")\n```',  # noqa
    ##################################################
    "## line_round_dot_setting property interface example": "## line_round_dot_setting属性のインターフェイス例",  # noqa
    ##################################################
    "The `line_round_dot_setting` property updates or gets the instance's line-round dot-style setting:": "`line_round_dot_setting`属性では丸ドット線のスタイル設定の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n)\nline.line_round_dot_setting = ap.LineRoundDotSetting(round_size=5, space_size=3)\n\nap.save_overall_html(dest_dir_path="line_line_round_dot_setting/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n)\nline.line_round_dot_setting = ap.LineRoundDotSetting(round_size=5, space_size=3)\n\nap.save_overall_html(dest_dir_path="line_line_round_dot_setting/")\n```',  # noqa
    ##################################################
    "## line_dash_dot_setting property interface example": "## line_dash_dot_setting属性のインターフェイス例",  # noqa
    ##################################################
    "The `line_dash_dot_setting` property updates or gets the instance's dash-dotted line style setting:": "`line_dash_dot_setting`属性では一点鎖線のスタイル設定の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nline.line_dash_dot_setting = ap.LineDashDotSetting(\n    dot_size=2, dash_size=5, space_size=2\n)\n\nap.save_overall_html(dest_dir_path="line_line_dash_dot_setting/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=50),\n    end_point=ap.Point2D(x=150, y=50),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nline.line_dash_dot_setting = ap.LineDashDotSetting(\n    dot_size=2, dash_size=5, space_size=2\n)\n\nap.save_overall_html(dest_dir_path="line_line_dash_dot_setting/")\n```',  # noqa
    ##################################################
    "## rotation_around_center property interface example": "## rotation_around_center属性のインターフェイス例",  # noqa
    ##################################################
    "The `rotation_around_center` property updates or gets the instance's rotation value (0 to 359) from the center point:": "`rotation_around_center`属性ではインスタンスの中央座標での回転量（0～359）の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.rotation_around_center += 1\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_rotation_around_center/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.rotation_around_center += 1\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_rotation_around_center/")\n```',  # noqa
    ##################################################
    "## set_rotation_around_point and get_rotation_around_point methods interface example": "## set_rotation_around_pointとget_rotation_around_pointメソッドのインターフェイス例",  # noqa
    ##################################################
    "The `set_rotation_around_point` method updates the instance's rotation value (0 to 359) from a specified point.": "`set_rotation_around_point`メソッドは指定された座標からのインスタンスの回転量（0～359）を更新します。",  # noqa
    ##################################################
    "Similarly, the `get_rotation_around_point` method gets the instance's rotation value (0 to 359) from a specified point:": "同様に、`get_rotation_around_point`メソッドでは指定された座標のインスタンスの回転量（0～359）を取得します:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nx: ap.Int = ap.Int(100)\ny: ap.Int = ap.Int(100)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rotation: ap.Int = line.get_rotation_around_point(x=x, y=y)\n    rotation += 1\n    line.set_rotation_around_point(rotation=rotation, x=x, y=y)\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_set_rotation_around_point/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\nx: ap.Int = ap.Int(100)\ny: ap.Int = ap.Int(100)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rotation: ap.Int = line.get_rotation_around_point(x=x, y=y)\n    rotation += 1\n    line.set_rotation_around_point(rotation=rotation, x=x, y=y)\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_set_rotation_around_point/")\n```',  # noqa
    ##################################################
    "## scale_x_from_center property interface example": "## scale_x_from_center属性のインターフェイス例",  # noqa
    ##################################################
    "The `scale_x_from_center` property updates or gets the instance's scale-x from the center point:": "`scale_x_from_center`属性ではインスタンスの中央座標でのX軸の拡縮値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    with ap.If(line.scale_x_from_center <= 0.001):\n        direction.value = 1\n    with ap.If(line.scale_x_from_center >= 2.0):\n        direction.value = -1\n    line.scale_x_from_center += direction * 0.005\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_x_from_center/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    with ap.If(line.scale_x_from_center <= 0.001):\n        direction.value = 1\n    with ap.If(line.scale_x_from_center >= 2.0):\n        direction.value = -1\n    line.scale_x_from_center += direction * 0.005\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_x_from_center/")\n```',  # noqa
    ##################################################
    "## scale_y_from_center property interface example": "## scale_y_from_center属性のインターフェイス例",  # noqa
    ##################################################
    "The `scale_y_from_center` property updates or gets the instance's scale-y from the center point:": "`scale_y_from_center`属性ではインスタンスの中央座標でのX軸の拡縮値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    with ap.If(line.scale_y_from_center <= 0.001):\n        direction.value = 1\n    with ap.If(line.scale_y_from_center >= 2.0):\n        direction.value = -1\n    line.scale_y_from_center += direction * 0.005\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_y_from_center/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    with ap.If(line.scale_y_from_center <= 0.001):\n        direction.value = 1\n    with ap.If(line.scale_y_from_center >= 2.0):\n        direction.value = -1\n    line.scale_y_from_center += direction * 0.005\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_y_from_center/")\n```',  # noqa
    ##################################################
    "## set_scale_x_from_point and get_scale_x_from_point methods interface example": "## set_scale_x_from_pointとget_scale_x_from_pointメソッドのインターフェイス例",  # noqa
    ##################################################
    "The `set_scale_x_from_point` method updates the instance's scale-x from a specified point.": "`set_scale_x_from_point`メソッドは指定されたX座標を基準としてX軸の拡縮値を更新します。",  # noqa
    ##################################################
    "Similarly, the `get_scale_x_from_point` method gets the instance's scale-x from a specified point:": "同様に、`get_scale_x_from_point`メソッドでは指定されたX座標を基準としたX軸の拡縮値を取得します:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\nx: ap.Int = ap.Int(150)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    scale: ap.Number = line.get_scale_x_from_point(x=x)\n    with ap.If(scale <= 0.001):\n        direction.value = 1\n    with ap.If(scale >= 2.0):\n        direction.value = -1\n    scale += direction * 0.005\n    line.set_scale_x_from_point(scale_x=scale, x=x)\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_x_from_point/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\nx: ap.Int = ap.Int(150)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    scale: ap.Number = line.get_scale_x_from_point(x=x)\n    with ap.If(scale <= 0.001):\n        direction.value = 1\n    with ap.If(scale >= 2.0):\n        direction.value = -1\n    scale += direction * 0.005\n    line.set_scale_x_from_point(scale_x=scale, x=x)\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_x_from_point/")\n```',  # noqa
    ##################################################
    "## set_scale_y_from_point and get_scale_y_from_point methods interface example": "## set_scale_y_from_pointとget_scale_y_from_pointメソッドのインターフェイス例",  # noqa
    ##################################################
    "The `set_scale_y_from_point` method updates the instance's scale-y from a specified point.": "`set_scale_y_from_point`メソッドは指定されたY座標を基準としてY軸の拡縮値を更新します。",  # noqa
    ##################################################
    "Similarly, the `get_scale_y_from_point` method gets the instance's scale-y from a specified point:": "同様に、`get_scale_y_from_point`メソッドでは指定されたY座標を基準としたY軸の拡縮値を取得します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\ny: ap.Int = ap.Int(60)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    scale: ap.Number = line.get_scale_y_from_point(y=y)\n    with ap.If(scale <= 0.001):\n        direction.value = 1\n    with ap.If(scale >= 2.0):\n        direction.value = -1\n    scale += direction * 0.005\n    line.set_scale_y_from_point(scale_y=scale, y=y)\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_y_from_point/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=30,\n)\ndirection: ap.Int = ap.Int(-1)\ny: ap.Int = ap.Int(60)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    scale: ap.Number = line.get_scale_y_from_point(y=y)\n    with ap.If(scale <= 0.001):\n        direction.value = 1\n    with ap.If(scale >= 2.0):\n        direction.value = -1\n    scale += direction * 0.005\n    line.set_scale_y_from_point(scale_y=scale, y=y)\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_scale_y_from_point/")\n```',  # noqa
    ##################################################
    "## flip_x property interface example": "## flip_x属性のインターフェイス例",
    ##################################################
    "The `flip_x` property updates or gets the instance's flip-x (reflecting state) boolean value:": "`flip_x`属性ではインスタンスのX軸の反転状況の真偽値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.flip_x = line.flip_x.not_\n\n\nap.Timer(on_timer, delay=1000).start()\nap.save_overall_html(dest_dir_path="line_flip_x/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.flip_x = line.flip_x.not_\n\n\nap.Timer(on_timer, delay=1000).start()\nap.save_overall_html(dest_dir_path="line_flip_x/")\n```',  # noqa
    ##################################################
    "Notes: Depending on the shape of the instance, this may be difficult to tell the difference between the x and y axes interfaces.": "特記事項: インスタンスの形状によってはこのインターフェイスはX軸とY軸の各インターフェイスで違いが分かりづらいケースが発生します。",  # noqa
    ##################################################
    "## flip_y property interface example": "## flip_y属性のインターフェイス例",
    ##################################################
    "The `flip_y` property updates or gets the instance's flip-y (reflecting state) boolean value:": "`flip_y`属性ではインスタンスのX軸の反転状況の真偽値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.flip_y = line.flip_y.not_\n\n\nap.Timer(on_timer, delay=1000).start()\nap.save_overall_html(dest_dir_path="line_flip_y/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.flip_y = line.flip_y.not_\n\n\nap.Timer(on_timer, delay=1000).start()\nap.save_overall_html(dest_dir_path="line_flip_y/")\n```',  # noqa
    ##################################################
    "Notes: Depending on the shape of the instance, this may be difficult to tell the difference between the x and y axes interfaces.": "特記事項: インスタンスの形状によってはこのインターフェイスはX軸とY軸の各インターフェイスで違いが分かりづらいケースが発生します。",  # noqa
    ##################################################
    "## skew_x property interface example": "## skew_x属性のインターフェイス例",
    ##################################################
    "The `skew_x` property updates or gets the instance's skew-x (distortion) value:": "`skew_x`属性ではインスタンスのX軸の歪みの値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.skew_x += 1\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_skew_x/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.skew_x += 1\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_skew_x/")\n```',  # noqa
    ##################################################
    "Notes: Depending on the shape of the instance, this may be difficult to tell the difference between the x and y axes interfaces.": "特記事項: インスタンスの形状によってはこのインターフェイスはX軸とY軸の各インターフェイスで違いが分かりづらいケースが発生します。",  # noqa
    ##################################################
    "## skew_y property interface example": "## skew_y属性のインターフェイス例",
    ##################################################
    "The `skew_y` property updates or gets the instance's skew-y (distortion) value:": "`skew_y`属性ではインスタンスのY軸の歪みの値の更新もしくは取得を行えます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.skew_y += 1\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_skew_y/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\nline: ap.Line = ap.Line(\n    start_point=ap.Point2D(x=50, y=40),\n    end_point=ap.Point2D(x=150, y=60),\n    line_color=ap.Color("#0af"),\n    line_thickness=3,\n)\n\n\ndef on_timer(e: ap.TimerEvent, options: dict) -> None:\n    """\n    The timer event handler.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    line.skew_y += 1\n\n\nap.Timer(on_timer, delay=ap.FPS.FPS_60).start()\nap.save_overall_html(dest_dir_path="line_skew_y/")\n```',  # noqa
    ##################################################
    "Notes: Depending on the shape of the instance, this may be difficult to tell the difference between the x and y axes interfaces.": "特記事項: インスタンスの形状によってはこのインターフェイスはX軸とY軸の各インターフェイスで違いが分かりづらいケースが発生します。",  # noqa
    ##################################################
    "## Line class constructor API": "## Line クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Create a line vector graphic.<hr>": "線のベクターグラフィックスを生成します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `start_point`: Points2D": "- `start_point`: Points2D",
    ##################################################
    "  - Line start point.": "  - 線の開始座標。",
    ##################################################
    "- `end_point`: Points2D": "- `end_point`: Points2D",
    ##################################################
    "  - Line end point.": "  - 線の終了座標。",
    ##################################################
    "- `line_color`: Color, default COLORLESS": "- `line_color`: Color, default COLORLESS",  # noqa
    ##################################################
    "  - A line-color to set.": "  - 設定する線の色。",
    ##################################################
    "- `line_alpha`: float or Number, default 1.0": "- `line_alpha`: float or Number, default 1.0",  # noqa
    ##################################################
    "  - A line-alpha to set.": "  - 設定する線の透明度。",
    ##################################################
    "- `line_thickness`: int or Int, default 1": "- `line_thickness`: int or Int, default 1",  # noqa
    ##################################################
    "  - A line-thickness (line-width) to set.": "  - 設定の線幅。",
    ##################################################
    "- `line_cap`: String or LineCaps or None, default None": "- `line_cap`: String or LineCaps or None, default None",  # noqa
    ##################################################
    "  - A line-cap setting to set.": "  - 設定する線の端のスタイル設定。",
    ##################################################
    "- `line_dot_setting`: LineDotSetting or None, default None": "- `line_dot_setting`: LineDotSetting or None, default None",  # noqa
    ##################################################
    "  - A dot setting to set.": "  - 設定する点線のスタイル設定。",
    ##################################################
    "- `line_dash_setting`: LineDashSetting or None, default None": "- `line_dash_setting`: LineDashSetting or None, default None",  # noqa
    ##################################################
    "  - A dash setting to set.": "  - 設定する破線のスタイル設定。",
    ##################################################
    "- `line_round_dot_setting`: LineRoundDotSetting or None, default None": "- `line_round_dot_setting`: LineRoundDotSetting or None, default None",  # noqa
    ##################################################
    "  - A round-dot setting to set.": "  - 設定する丸ドットのスタイル設定。",
    ##################################################
    "- `line_dash_dot_setting`: LineDashDotSetting or None, default None": "- `line_dash_dot_setting`: LineDashDotSetting or None, default None",  # noqa
    ##################################################
    "  - A dash-dot (1-dot chain) setting to set.": "  - 設定する一点鎖線のスタイル設定。",
    ##################################################
    "- `parent`: ChildMixIn or None, default None": "- `parent`: ChildMixIn or None, default None",  # noqa
    ##################################################
    "  - A parent instance to add this instance. If the specified value is None, this interface uses a stage instance.": "  - このインスタンスを追加する親のインスタンス。もしもNoneが指定された場合、このインスタンスはステージのインスタンスへと追加されます。",  # noqa
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> line: ap.Line = ap.Line(\n...     start_point=ap.Point2D(x=50, y=50),\n...     end_point=ap.Point2D(x=150, y=50),\n...     line_color=ap.Color("#ffffff"),\n...     line_thickness=3,\n... )\n>>> line.line_color\nColor("#ffffff")\n\n>>> line.line_thickness\nInt(3)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> line: ap.Line = ap.Line(\n...     start_point=ap.Point2D(x=50, y=50),\n...     end_point=ap.Point2D(x=150, y=50),\n...     line_color=ap.Color("#ffffff"),\n...     line_thickness=3,\n... )\n>>> line.line_color\nColor("#ffffff")\n\n>>> line.line_thickness\nInt(3)\n```',  # noqa
}



"""The mix-in class implementation for the `tick_max_num` value.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class TickMaxNumMixIn:
    _tick_max_num: Optional[Int]

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=True)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_tick_max_num(
        self,
        *,
        tick_max_num: Optional[Union[int, Int]],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set an initial tick max display number

        Parameters
        ----------
        tick_max_num : Optional[Union[int, Int]]
            A tick max display number.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if tick_max_num is not None and not isinstance(tick_max_num, Int):
            tick_max_num_: Optional[Int] = Int(
                tick_max_num, variable_name_suffix=variable_name_suffix
            )
        else:
            tick_max_num_ = tick_max_num
        self._tick_max_num = tick_max_num_



"""The class implementation for the outlined `alarm_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAlarmOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `alarm_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M10.04 6.29C10.66 6.11 11.32 6 12 6c3.86 0 7 3.14 7 7 0 .68-.11 1.34-.29 1.96l1.56 1.56c.47-1.08.73-2.27.73-3.52 0-4.97-4.03-9-9-9-1.25 0-2.44.26-3.53.72l1.57 1.57zm7.297-4.48l4.607 3.845-1.28 1.535-4.61-3.843zM3.02 2.1L1.61 3.51l1.37 1.37-.92.77 1.28 1.54 1.06-.88.8.8C3.83 8.69 3 10.75 3 13c0 4.97 4.03 9 9 9 2.25 0 4.31-.83 5.89-2.2l2.1 2.1 1.41-1.41L3.02 2.1zM12 20c-3.86 0-7-3.14-7-7 0-1.7.61-3.26 1.62-4.47l9.85 9.85C15.26 19.39 13.7 20 12 20zM7.48 3.73l.46-.38-1.28-1.54-.6.5z"/></svg>'  # noqa



"""The class implementation for the `view_in_ar` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialViewInArIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `view_in_ar` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M18.25 7.6l-5.5-3.18c-.46-.27-1.04-.27-1.5 0L5.75 7.6c-.46.27-.75.76-.75 1.3v6.35c0 .54.29 1.03.75 1.3l5.5 3.18c.46.27 1.04.27 1.5 0l5.5-3.18c.46-.27.75-.76.75-1.3V8.9c0-.54-.29-1.03-.75-1.3zM7 14.96v-4.62l4 2.32v4.61l-4-2.31zm5-4.03L8 8.61l4-2.31 4 2.31-4 2.32zm1 6.34v-4.61l4-2.32v4.62l-4 2.31zM7 2H3.5C2.67 2 2 2.67 2 3.5V7h2V4h3V2zm10 0h3.5c.83 0 1.5.67 1.5 1.5V7h-2V4h-3V2zM7 22H3.5c-.83 0-1.5-.67-1.5-1.5V17h2v3h3v2zm10 0h3.5c.83 0 1.5-.67 1.5-1.5V17h-2v3h-3v2z"/></svg>'  # noqa



"""The mix-in class implementation for the `tick_text_font_size` value.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class TickTextFontSizeMixIn:
    _tick_text_font_size: Int

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_tick_text_font_size(
        self,
        *,
        tick_text_font_size: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set an initial tick text font-size setting.

        Parameters
        ----------
        tick_text_font_size : Union[int, Int]
            A tick text font-size setting.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(tick_text_font_size, Int):
            tick_text_font_size_: Int = Int(
                tick_text_font_size, variable_name_suffix=variable_name_suffix
            )
        else:
            tick_text_font_size_ = tick_text_font_size
        self._tick_text_font_size = tick_text_font_size_



"""The class implementation for the `mic` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMicIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `mic` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M12 14c1.66 0 2.99-1.34 2.99-3L15 5c0-1.66-1.34-3-3-3S9 3.34 9 5v6c0 1.66 1.34 3 3 3zm5.3-3c0 3-2.54 5.1-5.3 5.1S6.7 14 6.7 11H5c0 3.41 2.72 6.23 6 6.72V21h2v-3.28c3.28-.48 6-3.3 6-6.72h-1.7z"/></svg>'  # noqa



"""The class implementation for the outlined `supervisor_account` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSupervisorAccountOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `supervisor_account` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M9 12c1.93 0 3.5-1.57 3.5-3.5S10.93 5 9 5 5.5 6.57 5.5 8.5 7.07 12 9 12zm0-5c.83 0 1.5.67 1.5 1.5S9.83 10 9 10s-1.5-.67-1.5-1.5S8.17 7 9 7zm.05 10H4.77c.99-.5 2.7-1 4.23-1 .11 0 .23.01.34.01.34-.73.93-1.33 1.64-1.81-.73-.13-1.42-.2-1.98-.2-2.34 0-7 1.17-7 3.5V19h7v-1.5c0-.17.02-.34.05-.5zm7.45-2.5c-1.84 0-5.5 1.01-5.5 3V19h11v-1.5c0-1.99-3.66-3-5.5-3zm1.21-1.82c.76-.43 1.29-1.24 1.29-2.18C19 9.12 17.88 8 16.5 8S14 9.12 14 10.5c0 .94.53 1.75 1.29 2.18.36.2.77.32 1.21.32s.85-.12 1.21-.32z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: variable_name_suffix.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# variable_name_suffix argument setting": "# variable_name_suffix の引数設定",
    ##################################################
    "This page explains the `variable_name_suffix` argument setting.": "このページでは`variable_name_suffix`の引数設定について説明します。",  # noqa
    ##################################################
    "## What argument is this?": "## 引数の概要",
    ##################################################
    "The `variable_name_suffix` argument changes an exported JavaScript's variable name suffix.": "`variable_name_suffix`引数は出力されるJavaScriptの変数名のサフィックスを変更します。",  # noqa
    ##################################################
    "This setting sometimes becomes useful when you want to debug an exported JavaScript code.": "この設定は出力されたJavaScriptに対してデバッグを行う際などに便利なことがあります。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each class has a `variable_name_suffix`, and you can set a suffix with its argument.": "apyscの各クラスは`variable_name_suffix`引数を持っており、その引数を使ってサフィックスを設定することができます。",  # noqa
    ##################################################
    "The following example sets the `my_int` suffix to the `ap.Int` instance:": "以下のコード例では`my_int`というサフィックスを`ap.Int`クラスのインスタンスへと設定しています:",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nap.Stage(stage_width=150, stage_height=150, stage_elem_id="stage")\nint_val: ap.Int = ap.Int(10, variable_name_suffix="my_int")\nap.trace(int_val)\n\nap.save_overall_html(\n    dest_dir_path="./variable_name_suffix_basic_usage_1/", minify=False\n)\n```': '```py\nimport apysc as ap\n\nap.Stage(stage_width=150, stage_height=150, stage_elem_id="stage")\nint_val: ap.Int = ap.Int(10, variable_name_suffix="my_int")\nap.trace(int_val)\n\nap.save_overall_html(\n    dest_dir_path="./variable_name_suffix_basic_usage_1/", minify=False\n)\n```',  # noqa
    ##################################################
    "In the exported JavaScript code, you can verify its `ap.Int` variable has the specified suffix, `_my_int`, as follows:": "出力されたJavaScriptコード内で、`ap.Int`クラスが変換されて作成された変数が以下のように指定された`_my_int`というサフィックスを持っていることを確認できます:",  # noqa
    ##################################################
    '```js\n  // ...\n  var i_9__my_int = 10;\n  console.log(i_9__my_int, "Called from: tmp.py, line number: 5");\n  // ...\n```': '```js\n  // ...\n  var i_9__my_int = 10;\n  console.log(i_9__my_int, "Called from: tmp.py, line number: 5");\n  // ...\n```',  # noqa
    ##################################################
    "Class attributes inherit a suffix value of its class argument's value.": "クラスの属性の値はそのクラスへ指定されたサフィックスの値を引き継ぎます。",  # noqa
    ##################################################
    "For example, if you set the `my_rectangle` value to the `variable_name_suffix` argument, its attributes (e.g., `x`, `fill_color`) also inherit the `my_rectangle` suffix:": "例えば`variable_name_suffix`引数に`my_rectangle`というサフィックスを指定した場合、そのクラスの属性（例 : `x`や`fill_color`など）は同様に`my_rectangle`というサフィックスを持つようになります:",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nap.Stage(stage_width=150, stage_height=150, stage_elem_id="stage")\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n    variable_name_suffix="my_rectangle",\n)\n\nap.save_overall_html(\n    dest_dir_path="./variable_name_suffix_basic_usage_2/", minify=False\n)\n```': '```py\nimport apysc as ap\n\nap.Stage(stage_width=150, stage_height=150, stage_elem_id="stage")\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n    variable_name_suffix="my_rectangle",\n)\n\nap.save_overall_html(\n    dest_dir_path="./variable_name_suffix_basic_usage_2/", minify=False\n)\n```',  # noqa
    ##################################################
    "In that case, each attribute also has an attribute identifier suffix, such as the `x`, `width`:": "このようなケースでは`x`や`width`などの属性の識別用のサフィックスも同様に設定されます:",  # noqa
    ##################################################
    '```js\n  var i_9__my_rectangle__x = 50;\n  var i_10__my_rectangle__y = 50;\n  var i_12__my_rectangle__width = 50;\n  // ...\n  var rect_1__my_rectangle = stage\n    .rect(i_16__my_rectangle__width, i_17__my_rectangle__height)\n    .attr({\n      fill: s_1__my_rectangle__fill_color,\n      "fill-opacity": n_2__my_rectangle__fill_alpha,\n      "stroke-width": i_15__my_rectangle__line_thickness,\n      "stroke-opacity": n_3__my_rectangle__line_alpha,\n      x: i_9__my_rectangle__x,\n      y: i_10__my_rectangle__y,\n    });\n  // ...\n```': '```js\n  var i_9__my_rectangle__x = 50;\n  var i_10__my_rectangle__y = 50;\n  var i_12__my_rectangle__width = 50;\n  // ...\n  var rect_1__my_rectangle = stage\n    .rect(i_16__my_rectangle__width, i_17__my_rectangle__height)\n    .attr({\n      fill: s_1__my_rectangle__fill_color,\n      "fill-opacity": n_2__my_rectangle__fill_alpha,\n      "stroke-width": i_15__my_rectangle__line_thickness,\n      "stroke-opacity": n_3__my_rectangle__line_alpha,\n      x: i_9__my_rectangle__x,\n      y: i_10__my_rectangle__y,\n    });\n  // ...\n```',  # noqa
}



"""The class implementation for the outlined `shuffle` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialShuffleOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `shuffle` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M10.59 9.17L5.41 4 4 5.41l5.17 5.17 1.42-1.41zM14.5 4l2.04 2.04L4 18.59 5.41 20 17.96 7.46 20 9.5V4h-5.5zm.33 9.41l-1.41 1.41 3.13 3.13L14.5 20H20v-5.5l-2.04 2.04-3.13-3.13z"/></svg>'  # noqa



"""Enum definition for the y-axis label position.
"""

from enum import Enum


class YAxisLabelPosition(Enum):
    OUTER_TOP = "outer_top"
    OUTER_CENTER = "outer_center"
    OUTER_BOTTOM = "outer_bottom"
    INNER_TOP = "inner_top"
    INNER_CENTER = "inner_center"
    INNER_RIGHT = "inner_right"



"""The class implementation for the `find_in_page` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFindInPageIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `find_in_page` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M20 19.59V8l-6-6H6c-1.1 0-1.99.9-1.99 2L4 20c0 1.1.89 2 1.99 2H18c.45 0 .85-.15 1.19-.4l-4.43-4.43c-.8.52-1.74.83-2.76.83-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5c0 1.02-.31 1.96-.83 2.75L20 19.59zM9 13c0 1.66 1.34 3 3 3s3-1.34 3-3-1.34-3-3-3-3 1.34-3 3z"/></svg>'  # noqa



"""Class implementation for the dictionary value.
"""

from typing import Any
from typing import Dict
from typing import Generic
from typing import Optional
from typing import TypeVar
from typing import Union

from typing_extensions import final

from apysc._event.custom_event_mixin import CustomEventMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._type.any_value import AnyValue
from apysc._type.boolean import Boolean
from apysc._type.copy_mixin import CopyMixIn
from apysc._type.dictionary_structure import DictionaryStructure
from apysc._type.expression_string import ExpressionString
from apysc._type.initial_substitution_exp_mixin import InitialSubstitutionExpMixIn
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.py_builtin_iter_disabling_mixin import PyBuiltInIterDisablingMixIn
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.string import String
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos

DefaultType = TypeVar("DefaultType")
_BuiltinKeys = Union[str, int, float]
_Key = TypeVar("_Key")
_Value = TypeVar("_Value")


class Dictionary(
    CustomEventMixIn,
    CopyMixIn["Dictionary"],
    RevertMixIn,
    DictionaryStructure,
    VariableNameSuffixMixIn,
    InitialSubstitutionExpMixIn,
    PyBuiltInIterDisablingMixIn,
    InitializeWithBaseValueInterface,
    Generic[_Key, _Value],
):
    """
    Dictionary class for the apysc library.

    References
    ----------
    - Dictionary
        - https://simon-ritchie.github.io/apysc/en/dictionary.html
    - Dictionary class generic type settings
        - https://simon-ritchie.github.io/apysc/en/dictionary_generic.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})
    >>> dictionary
    Dictionary({'a': 10})

    >>> dictionary["a"]
    10

    >>> dictionary["b"] = 20
    >>> dictionary["b"]
    20

    >>> dictionary.length
    Int(2)

    >>> value_1: int = dictionary.get("c", default=0)
    >>> value_1
    0
    """

    _initial_value: Union[Dict[_Key, _Value], "Dictionary"]
    _value: Dict[_Key, _Value]

    @arg_validation_decos.is_acceptable_dictionary_value(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @arg_validation_decos.is_builtin_boolean(arg_position_index=3)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        value: Union[Dict[_Key, _Value], "Dictionary"],
        *,
        variable_name_suffix: str = "",
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        Dictionary class for the apysc library.

        Parameters
        ----------
        value : dict or Dictionary
            Initial dictionary value.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. This class uses this option internally.

        References
        ----------
        - Dictionary
            - https://simon-ritchie.github.io/apysc/en/dictionary.html
        - Dictionary class generic type settings
            - https://simon-ritchie.github.io/apysc/en/dictionary_generic.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})
        >>> dictionary
        Dictionary({'a': 10})

        >>> dictionary["a"]
        10

        >>> dictionary["b"] = 20
        >>> dictionary["b"]
        20
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            self._variable_name_suffix = variable_name_suffix
            TYPE_NAME: str = var_names.DICTIONARY
            self._initial_value = value
            self._type_name = TYPE_NAME
            self._value = self._get_dict_value(value=value)
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=TYPE_NAME
            )
            self._append_constructor_expression()

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append constructor expression.
        """
        from apysc._expression import expression_data_util

        expression: str = self._create_initial_substitution_expression()
        expression = f"var {expression}"
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _create_initial_substitution_expression(self) -> str:
        """
        Create an initial value's substitution expression string.

        Returns
        -------
        expression : str
            Created expression string.
        """
        from apysc._type import value_util

        value_str: str = value_util.get_value_str_for_expression(
            value=self._initial_value
        )
        expression: str = f"{self.variable_name} = {value_str};"
        return expression

    @final
    def _get_dict_value(
        self, *, value: Union[Dict[_Key, _Value], "Dictionary"]
    ) -> Dict[_Key, _Value]:
        """
        Get a dict value from a specified value.

        Parameters
        ----------
        value : dict or Dictionary
            Specified dictionary value.

        Returns
        -------
        dict_val : dict
            Converted dict value.
        """
        if isinstance(value, Dictionary):
            return value._value
        return value  # type: ignore

    @property
    @add_debug_info_setting(module_name=__name__)
    def value(self) -> Union[Dict[_Key, _Value], "Dictionary"]:
        """
        Get a current dict value.

        Returns
        -------
        value : dict
            Current dict value.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> dictionary: ap.Dictionary = ap.Dictionary({})
        >>> dictionary.value = {"a": 10}
        >>> dictionary.value
        {'a': 10}
        """
        return self._value

    @value.setter
    @arg_validation_decos.is_acceptable_dictionary_value(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def value(self, value: Union[Dict[_Key, _Value], "Dictionary"]) -> None:
        """
        Set dictionary value.

        Parameters
        ----------
        value : dict or Dictionary.
            Dictionary value to set.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa
        """
        self._value = self._get_dict_value(value=value)
        self._append_value_setter_expression(value=value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_value_setter_expression(
        self, *, value: Union[Dict[_Key, _Value], "Dictionary"]
    ) -> None:
        """
        Append value's setter expression.

        Parameters
        ----------
        value : dict or Dictionary.
            Dictionary value to set.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        value_str: str = value_util.get_value_str_for_expression(value=value)
        expression: str = f"{self.variable_name} = {value_str};"
        expression_data_util.append_js_expression(expression=expression)

    _value_snapshot: Optional[Dict[str, Dict[_Key, _Value]]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapthot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_value_snapshot",
            value={**self._value},
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._value = self._get_snapshot_val_if_exists(
            current_value=self._value,
            snapshot_dict=self._value_snapshot,
            snapshot_name=snapshot_name,
        )

    def __str__(self) -> str:
        """
        String conversion method.

        Returns
        -------
        string : str
            Converted value string.
        """
        if not hasattr(self, "_value"):
            return "{}"
        return str(self._value)

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, "_value"):
            repr_str: str = "Dictionary({})"
        else:
            repr_str = f"Dictionary({self._value})"
        return repr_str

    @property
    @add_debug_info_setting(module_name=__name__)
    def length(self) -> Int:
        """
        Get length of this dictionary values.

        Returns
        -------
        length : Int
            This dictionary value's length.

        References
        ----------
        - Dictionary length interface
            - https://simon-ritchie.github.io/apysc/en/dictionary_length.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 1, "b": 2})
        >>> dictionary.length
        Int(2)
        """

        length: Int = Int(len(self._value))
        self._append_length_expression(length=length)
        return length

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_length_expression(self, *, length: Int) -> None:
        """
        Append length method expression.

        Parameters
        ----------
        length : Int
            Created length Int variable.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{length.variable_name} = " f"Object.keys({self.variable_name}).length;"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def __len__(self) -> None:
        """
        This method is disabled and can't use from a
        Dictionary instance.
        """
        raise Exception(
            "Dictionary instance can't apply len function."
            " Please use length property instead."
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def __getitem__(self, key: Union[_Key, ExpressionString]) -> _Value:
        """
        Get a specified key's single value.

        Parameters
        ----------
        key : _Key
            Dictionary key.

        Returns
        -------
        value : *
            Specified key's value.
        """
        from apysc._type.any_value import AnyValue

        self._validate_key_type_is_str_or_numeric(key=key)
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        has_key: bool = key_ in self._value
        if has_key:
            value: Any = self._value[key_]  # type: ignore
        else:
            value = AnyValue(None)
        self._append_getitem_expression(key=key, value=value)
        return value

    @final
    def _get_builtin_type_key(
        self, *, key: Union[_Key, ExpressionString]
    ) -> _BuiltinKeys:
        """
        Get a built-in type's key (str, int, or float) from
        a specified key.

        Parameters
        ----------
        key : _Key
            Target key value (including String, Int, and Number).

        Returns
        -------
        key : str or int or float
            Built-in type's key.
        """
        if isinstance(key, ExpressionString):
            return key.value
        if isinstance(key, (String, Int, Number)):
            return key._value
        key_: _BuiltinKeys = key  # type: ignore
        return key_

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_getitem_expression(
        self, *, key: Union[_Key, ExpressionString], value: Any
    ) -> None:
        """
        Append __getitem__ expression.

        Parameters
        ----------
        key : _Key
            Dictionary key.
        value : *
            Specified key's value.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util
        from apysc._type.any_value import AnyValue

        if not isinstance(value, VariableNameMixIn):
            value = AnyValue(None)
        key_str: str = value_util.get_value_str_for_expression(value=key)
        expression: str = (
            f"var {value.variable_name} = " f"{self.variable_name}[{key_str}];"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _validate_key_type_is_str_or_numeric(
        self, *, key: Union[_Key, ExpressionString]
    ) -> None:
        """
        Validate whether a key's value type is acceptable
        (str or int or float) or not.

        Parameters
        ----------
        key : _Key
            Dictionary key to check.

        Raises
        ------
        ValueError
            If a key's type is not String, Int, Number, str, int,
            or float.
        """
        if isinstance(key, (str, String, int, Int, float, Number, ExpressionString)):
            return
        raise ValueError(
            f"Unsupported key type is specified: {type(key)}, {key}"
            f"\nSuppoting types are: str, String, int, Int"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def __setitem__(self, key: Union[_Key, ExpressionString], value: _Value) -> None:
        """
        Set value to a specified key position.

        Parameters
        ----------
        key : _Key
            Dictionary key to set value.
        value : *
            Any value to set.
        """
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        self._value[key_] = value  # type: ignore
        self._append_setitem_expression(key=key, value=value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_setitem_expression(
        self, *, key: Union[_Key, ExpressionString], value: _Value
    ) -> None:
        """
        Append __setitem__ method expression.

        Parameters
        ----------
        key : _Key
            Dictionary key to check.
        value : *
            Any value to set.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        key_str: str = value_util.get_value_str_for_expression(value=key)
        value_str: str = value_util.get_value_str_for_expression(value=value)
        expression: str = f"{self.variable_name}[{key_str}] = {value_str};"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __delitem__(self, key: Union[_Key, ExpressionString]) -> None:
        """
        Delete specified key's value from a dictionary.

        Parameters
        ----------
        key : _Key
            Dictionary key to delete.
        """
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        if key_ in self._value:
            del self._value[key_]  # type: ignore
        self._append_delitem_expression(key=key)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_delitem_expression(self, *, key: Union[_Key, ExpressionString]) -> None:
        """
        Append __delitem__ method expression.

        Parameters
        ----------
        key : _Key
            Dictionary key to delete.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        key_str: str = value_util.get_value_str_for_expression(value=key)
        expression: str = f"delete {self.variable_name}[{key_str}];"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __eq__(self, other: Any) -> Any:
        """
        Equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. dict or Dictionary
            types are acceptable.

        Returns
        -------
        result : Boolean
            Comparison result.
        """

        if isinstance(other, Dictionary):
            result: Boolean = Boolean(self._value == other._value)
        else:
            result = Boolean(self._value == other)
            if isinstance(other, dict):
                other = Dictionary(other)
        if isinstance(other, VariableNameMixIn):
            self._append_eq_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_eq_expression(
        self, *, result: Boolean, other: VariableNameMixIn
    ) -> None:
        """
        Append an __eq__ expression.

        Parameters
        ----------
        result : Boolean
            A result boolean value.
        other : Dictionary
            The Dictionary's other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"_.isEqual({self.variable_name}, {other.variable_name});"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __ne__(self, other: Any) -> Any:
        """
        Noe equal comparison method.

        Parameters
        ----------
        other : *
            The other value to compare. dict or Dictionary types
            are acceptable.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        if isinstance(other, dict):
            other = Dictionary(other)
        result: Boolean = self == other
        result = result.not_
        if isinstance(other, VariableNameMixIn):
            self._append_ne_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_ne_expression(
        self, *, result: Boolean, other: VariableNameMixIn
    ) -> None:
        """
        Append a __ne__ expression.

        Parameters
        ----------
        result : Boolean
            A result boolean value.
        other : Dictionary
            The Dictionary's other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"!_.isEqual({self.variable_name}, {other.variable_name});"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def get(
        self,
        key: Union[_Key, ExpressionString],
        *,
        default: Optional[DefaultType] = None,
    ) -> DefaultType:
        """
        Get a specified key dictionary value. If this
        dictionary hasn't a specified key, this interface
        returns a default value.

        Parameters
        ----------
        key : _Key
            Target key.
        default : DefaultType or None, optional
            Any default value.

        Returns
        -------
        result_value : Any
            Extracted value or a default value.

        References
        ----------
        - Dictionary get interface
            - https://simon-ritchie.github.io/apysc/en/dictionary_get.html

        Examples
        --------
        >>> from typing import Optional
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})
        >>> value_1: Optional[int] = dictionary.get("a")
        >>> value_1
        10

        >>> value_2: Optional[int] = dictionary.get("b")
        >>> print(value_2)
        None

        >>> value_3: int = dictionary.get("c", default=0)
        >>> value_3
        0
        """
        key_: _BuiltinKeys = self._get_builtin_type_key(key=key)
        if key_ in self._value:
            result_value: Any = self._value[key_]  # type: ignore
        else:
            result_value = default
        self._append_get_expression(key=key, result_value=result_value, default=default)
        return result_value

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_get_expression(
        self, *, key: Union[_Key, ExpressionString], result_value: Any, default: Any
    ) -> None:
        """
        Append the `get` method expression.

        Parameters
        ----------
        key : _Key
            Target key.
        result_value : Any
            Extracted value or a default value.
        default : Any
            Any default value. Basic apysc types (e.g.,
            Int, Number, String, and so on) are necessary.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        key_str: str = value_util.get_value_str_for_expression(value=key)
        if not isinstance(result_value, VariableNameMixIn):
            result_value = AnyValue(value=None)
        result_value_str: str = value_util.get_value_str_for_expression(
            value=result_value
        )
        default_value_str: str = value_util.get_value_str_for_expression(value=default)
        expression: str = (
            f"if ({key_str} in {self.variable_name}) {{"
            f"\n  {result_value_str} = {self.variable_name}[{key_str}];"
            "\n}else {"
            f"\n  {result_value_str} = {default_value_str};"
            "\n}"
        )
        expression_data_util.append_js_expression(expression=expression)

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "Dictionary":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        dict_val : Dictionary
            An initialized dictionary value.
        """
        return Dictionary({})



"""The class implementation for the `False_` constant.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.bool_const_mixin import BoolConstMixIn
from apysc._type.boolean import Boolean


class _False(
    BoolConstMixIn,
    Boolean,
):
    _value: bool = False

    @final
    @add_debug_info_setting(module_name=__name__)
    def __init__(self) -> None:
        """
        The class implementation for the `False_` constant.

        References
        ----------
        - True_ and False_ constants
            - https://simon-ritchie.github.io/apysc/en/true_and_false.html
        """
        super(_False, self).__init__(
            value=False,
            variable_name_suffix="false",
        )



"""This module is for the translation mapping data of the
following document:

Document file: graphics.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics class": "# Graphics クラス",
    ##################################################
    "This page explains the `Graphics` class.": "このページでは`Graphics`クラスについて説明します。",
    ##################################################
    "## What is Graphics?": "## Graphics クラスの概要",
    ##################################################
    "The `Graphics` is the class to handle each vector graphics interface. This interface has the draw rectangle interface, draw line interface, or something else.": "`Graphics`クラスは各ベクターグラフィックスの描画のインターフェイスを扱うクラスです。このインターフェイスには四角の描画や線の描画など様々なインターフェイスが存在します。",  # noqa
    ##################################################
    "The `Sprite` or other `DisplayObject` instances instantiate this class instance.": "`Sprite`クラスなどのインスタンスはこの`Graphics`クラスのインスタンスを内部で生成します。",  # noqa
    ##################################################
    "## Call interfaces from sprite instance": "## Sprite のインスタンスを経由した各インターフェイスの呼び出し",
    ##################################################
    "Sprite (object container) instance has the `graphics` attribute to call each drawing interface with this attribute.": "Sprite のインスタンスは`graphics`属性を持っており、その属性を使用して各描画のインターフェイスを呼び出すことができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=180,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the white border and cyan color rectangle.\nsprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n# Draw the magenta color polyline.\nsprite.graphics.begin_fill(color=ap.COLORLESS)\nsprite.graphics.line_style(color=ap.Color("#f0a"), thickness=5)\nsprite.graphics.move_to(x=150, y=50)\nsprite.graphics.line_to(x=200, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\n# Draw the dashed line.\nsprite.graphics.draw_dashed_line(\n    x_start=50, y_start=130, x_end=200, y_end=130, dash_size=10, space_size=5\n)\n\nap.save_overall_html(dest_dir_path="graphics_call_interfaces_from_sprite_instance/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=180,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the white border and cyan color rectangle.\nsprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n# Draw the magenta color polyline.\nsprite.graphics.begin_fill(color=ap.COLORLESS)\nsprite.graphics.line_style(color=ap.Color("#f0a"), thickness=5)\nsprite.graphics.move_to(x=150, y=50)\nsprite.graphics.line_to(x=200, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\n# Draw the dashed line.\nsprite.graphics.draw_dashed_line(\n    x_start=50, y_start=130, x_end=200, y_end=130, dash_size=10, space_size=5\n)\n\nap.save_overall_html(dest_dir_path="graphics_call_interfaces_from_sprite_instance/")\n```',  # noqa
    ##################################################
    "## Return values": "## 各返却値について",
    ##################################################
    "Each interface returns created graphic instances (e.g., `Rectangle`\\, `Polyline`\\, and so on). These instances have the basic `DisplayObject` attributes and methods, like x, y, fill_alpha, visible, or something else.": "各インターフェイスは`Rectangle`や`Polyline`クラスなどの生成されたグラフィックスのインスタンスを返却します。これらのインスタンスはxやy、fill_alpha、visibleなどの基本的な`DisplayObject`クラスの各属性やメソッドなどを持っています。",  # noqa
    ##################################################
    "For example, you can set an event and coordinate's updating to these instances, as follows:": "例えば以下のコード例のようにこれらのインスタンスに対してイベントを設定して座標の更新処理などを設定することができます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Update the coordinates, fill alpha, and fill color.\n    rectangle.x = ap.Number(100)\n    rectangle.y = ap.Number(100)\n    rectangle.fill_alpha = ap.Number(0.5)\n    rectangle.fill_color = ap.Color("#f0a")\n\n\n# drew_rect interface will return Rectangle instance.\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n# Bind click event to the rectangle.\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="graphics_return_values/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Update the coordinates, fill alpha, and fill color.\n    rectangle.x = ap.Number(100)\n    rectangle.y = ap.Number(100)\n    rectangle.fill_alpha = ap.Number(0.5)\n    rectangle.fill_color = ap.Color("#f0a")\n\n\n# drew_rect interface will return Rectangle instance.\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n# Bind click event to the rectangle.\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="graphics_return_values/")\n```',  # noqa
    ##################################################
    "If you click the following rectangle, that rectangle changes x and y coordinates, fill color, and alpha (opacity) values.": "もし以下の四角をクリックすると座標値や塗りの色、透明度などの値が更新されます。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Graphics class begin_fill interface](graphics_begin_fill.md)": "- [Graphics クラスの begin_fill (塗りの設定)のインターフェイス](jp_graphics_begin_fill.md)",  # noqa
    ##################################################
    "- [Graphics class line_style interface](graphics_line_style.md)": "- [Graphics クラスの line_style (線のスタイル設定)のインターフェイス](jp_graphics_line_style.md)",  # noqa
    ##################################################
    "- [Graphics class draw_rect interface](graphics_draw_rect.md)": "- [Graphics クラスの draw_rect (四角の描画)のインターフェイス](jp_graphics_draw_rect.md)",  # noqa
    ##################################################
    "- [Graphics class draw_round_rect interface](graphics_draw_round_rect.md)": "- [Graphics クラスの draw_round_rect (角丸の四角の描画)のインターフェイス](jp_graphics_draw_round_rect.md)",  # noqa
    ##################################################
    "- [Graphics class draw_circle interface](graphics_draw_circle.md)": "- [Graphics クラスの draw_circle (円の描画)のインターフェイス](jp_graphics_draw_circle.md)",  # noqa
    ##################################################
    "- [Graphics class draw_ellipse interface](graphics_draw_ellipse.md)": "- [Graphics クラスの draw_ellipse (楕円描画) のインターフェイス](jp_graphics_draw_ellipse.md)",  # noqa
    ##################################################
    "- [Graphics class move_to and line_to interfaces](graphics_move_to_and_line_to.md)": "- [Graphics クラスの move_to (線の描画位置の変更)と line_to (指定座標への線の描画)のインターフェイス](jp_graphics_move_to_and_line_to.md)",  # noqa
    ##################################################
    "- [Graphics class draw_line interface](graphics_draw_line.md)": "- [Graphics クラスの draw_line (線の描画)のインターフェイス](jp_graphics_draw_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dotted_line interface](graphics_draw_dotted_line.md)": "- [Graphics クラスの draw_dotted_line (点線の描画)のインターフェイス](jp_graphics_draw_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dashed_line interface](graphics_draw_dashed_line.md)": "- [Graphics クラスの draw_dashed_line (破線の描画)のインターフェイス](jp_graphics_draw_dashed_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_round_dotted_line interface](graphics_draw_round_dotted_line.md)": "- [Graphics クラスの draw_round_dotted_line (点線(丸)の描画)のインターフェイス](jp_graphics_draw_round_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_dash_dotted_line interface](graphics_draw_dash_dotted_line.md)": "- [Graphics クラスの draw_dash_dotted_line (一点鎖線の描画)のインターフェイス](jp_graphics_draw_dash_dotted_line.md)",  # noqa
    ##################################################
    "- [Graphics class draw_polygon interface](graphics_draw_polygon.md)": "- [Graphics クラスの draw_polygon (多角形描画)のインターフェイス](jp_graphics_draw_polygon.md)",  # noqa
}



"""Class implementation for the clamp-related mix-in.
"""

from typing import TypeVar
from typing import Union
from typing import cast

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._validation import arg_validation_decos

_ValueType = TypeVar("_ValueType", Int, Number)


class ClampMixIn:
    @classmethod
    @final
    @arg_validation_decos.is_apysc_int_or_number(arg_position_index=1)
    @arg_validation_decos.is_apysc_int_or_number(arg_position_index=2)
    @arg_validation_decos.is_apysc_int_or_number(arg_position_index=3)
    @add_debug_info_setting(module_name=__name__)
    def clamp(
        cls, *, value: _ValueType, min_: _ValueType, max_: _ValueType
    ) -> _ValueType:
        """
        Sets the value within a specified minimum and maximum range.
        If the value is less than the minimum, this method returns the minimum value.
        If the value is greater than the maximum, this method returns the
        maximum value.

        Parameters
        ----------
        value : _ValueType
            Target `Int` or `Number` value.
        min_ : _ValueType
            Minimum value.
        max_ : _ValueType
            Maximum value.

        Returns
        -------
        result : _ValueType
            Clamped value.

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()

        >>> value: ap.Int = ap.Int(5)
        >>> value = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(20))
        >>> value
        Int(10)

        >>> value = ap.Int(25)
        >>> value = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(20))
        >>> value
        Int(20)
        """
        from apysc._math.math import Math
        from apysc._type.array import Array

        is_int_value: bool = isinstance(value, Int)

        py_value: Union[int, float] = value._value
        py_value = min(py_value, max_._value)
        py_value = max(py_value, min_._value)

        result: _ValueType = cast(_ValueType, Math.min(values=Array([value, max_])))
        result = cast(_ValueType, Math.max(values=Array([result, min_])))

        if is_int_value:
            result = cast(_ValueType, Math.trunc(value=result))

        result._value = py_value  # type: ignore
        return result



"""The class implementation for the outlined `videocam_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVideocamOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `videocam_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M9.56 8l-2-2-4.15-4.14L2 3.27 4.73 6H4c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h12c.21 0 .39-.08.55-.18L19.73 21l1.41-1.41-8.86-8.86L9.56 8zM5 16V8h1.73l8 8H5zm10-8v2.61l6 6V6.5l-4 4V7c0-.55-.45-1-1-1h-5.61l2 2H15z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_line_dash_setting.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase line_dash_setting interface": "# GraphicsBase クラスの line_dash_setting インターフェイス",  # noqa
    ##################################################
    "This page explains the `GraphicsBase` class `line_dash_setting` property interface.": "このページでは`GraphicsBase`クラスの`line_dash_setting`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `line_dash_setting` property interface updates or gets the instance's current line dash setting.": "`line_dash_setting`",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter or setter interface value becomes (or requires) the `LineDashSetting` instance value.": "getterとsetterのインターフェイスの値は`LineDashSetting`インスタンスの値となります。",  # noqa
    ##################################################
    "The following example sets the 10-pixel dash size and 3-pixel space size to the line:": "以下のコード例では線に対して10pxの破線と3pxの破線間の空白のスペースを設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=100,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=200, y_end=50)\nline.line_dash_setting = ap.LineDashSetting(dash_size=10, space_size=3)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dash_setting_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=100,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=200, y_end=50)\nline.line_dash_setting = ap.LineDashSetting(dash_size=10, space_size=3)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dash_setting_basic_usage/")\n```',  # noqa
    ##################################################
    "## Delete setting": "## 設定の削除",
    ##################################################
    "The `delete_line_dash_setting` interface deletes this line setting.": "`delete_line_dash_setting`インターフェイスはこの線の設定を削除します。",  # noqa
    ##################################################
    "In the following example, if you click the rectangle, the handler deletes the line setting:": "以下の例では四角をクリックするとハンドラ内の処理で線の設定を削除しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler for the click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Rectangle]\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.delete_line_dash_setting()\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#666"),\n    line_color=ap.Color("#fff"),\n    line_thickness=2,\n    line_dash_setting=ap.LineDashSetting(dash_size=4, space_size=2),\n)\nrectangle.click(handler=on_click)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dash_setting_delete_setting/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler for the click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Rectangle]\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.delete_line_dash_setting()\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#666"),\n    line_color=ap.Color("#fff"),\n    line_thickness=2,\n    line_dash_setting=ap.LineDashSetting(dash_size=4, space_size=2),\n)\nrectangle.click(handler=on_click)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dash_setting_delete_setting/")\n```',  # noqa
    ##################################################
    "## line_dash_setting property API": "## line_dash_setting 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current line dash setting.<hr>": "現在の線の破線のスタイル設定を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line_dash_setting`: LineDashSetting or None": "- `line_dash_setting`: LineDashSetting or None",  # noqa
    ##################################################
    "  - Line dash setting.": "  - 線の破線のスタイル設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_dash_setting = ap.LineDashSetting(dash_size=5, space_size=2)\n>>> line.line_dash_setting.dash_size\nInt(5)\n\n>>> line.line_dash_setting.space_size\nInt(2)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_dash_setting = ap.LineDashSetting(dash_size=5, space_size=2)\n>>> line.line_dash_setting.dash_size\nInt(5)\n\n>>> line.line_dash_setting.space_size\nInt(2)\n```',  # noqa
    ##################################################
    "## delete_line_dash_setting API": "## delete_line_dash_setting のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Delete a current line dash setting.": "現在の破線の設定を削除します。",
}



"""The mix-in class implementation for the font-size setting.
"""

from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class TextFontSizeCssMixIn(
    AttrLinkingMixIn,
):
    _font_size: Int

    def _initialize_font_size(self) -> None:
        """
        Initialize the `_font_size` attribute.
        """
        if hasattr(self, "_font_size"):
            return
        self._font_size = Int(16)

    @property
    def font_size(self) -> Int:
        """
        Get a text's font size.

        Returns
        -------
        font_size : Int
            A text font size.

        References
        ----------
        - Text font_size property
            - https://simon-ritchie.github.io/apysc/en/text_font_size.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=350,
        ...     stage_height=250,
        ...     stage_elem_id="stage",
        ... )
        >>> text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Example of font-size = 32. Lorem ipsum dolor sit amet, "
        ...     "consectetur adipiscing elit, sed do eiusmod tempor incididunt.",
        ...     width=300,
        ...     fill_color=ap.Color("#00aaff"),
        ...     x=25,
        ...     y=25,
        ... )
        >>> text.font_size = ap.Int(32)
        >>> text.font_size
        Int(32)
        """
        self._initialize_font_size()
        return self._font_size.copy()

    @font_size.setter
    @arg_validation_decos.is_apysc_integer(arg_position_index=1)
    def font_size(self, value: Int) -> None:
        """
        Set a text's font size.

        Parameters
        ----------
        value : Int
            A text font size.

        References
        ----------
        - Text font_size property
            - https://simon-ritchie.github.io/apysc/en/text_font_size.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=350,
        ...     stage_height=250,
        ...     stage_elem_id="stage",
        ... )
        >>> text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Example of font-size = 32. Lorem ipsum dolor sit amet, "
        ...     "consectetur adipiscing elit, sed do eiusmod tempor incididunt.",
        ...     width=300,
        ...     fill_color=ap.Color("#00aaff"),
        ...     x=25,
        ...     y=25,
        ... )
        >>> text.font_size = ap.Int(32)
        >>> text.font_size
        Int(32)
        """
        from apysc._display.css_interface import CssInterface
        from apysc._validation import display_validation

        interface: CssInterface = display_validation.validate_css_interface(
            instance=self
        )
        self._font_size = value
        interface.set_css(
            name="font-size",
            value=self._font_size.to_string() + "px",
        )

        self._append_applying_new_attr_val_exp(
            new_attr=self._font_size, attr_name="font_size"
        )
        self._append_attr_to_linking_stack(attr=self._font_size, attr_name="font_size")



"""The class implementation for the `visibility` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVisibilityIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `visibility` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M12 4.5C7 4.5 2.73 7.61 1 12c1.73 4.39 6 7.5 11 7.5s9.27-3.11 11-7.5c-1.73-4.39-6-7.5-11-7.5zM12 17c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5zm0-8c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"/></svg>'  # noqa



"""This module is for the attribute conversion interface from
built-in value to apysc value.
"""

from typing import Union

from typing_extensions import final

from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)


class AttrToApyscValFromBuiltinMixIn:
    @final
    def _get_copied_int_from_builtin_val(
        self, *, integer: Union[int, Int], attr_identifier: str
    ) -> Int:
        """
        Get a copied Int value from a Python built-in int
        and set an attribute's variable name suffix.

        Parameters
        ----------
        integer : Union[int, Int]
            Target integer value.
        attr_identifier : str
            Attribute identifier string (e.g., `x`).

        Returns
        -------
        copied : Int
            Copied Int value.
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_int_from_builtin_val,
        )

        suffix: str = _get_variable_name_suffix(
            instance=self, attr_identifier=attr_identifier
        )
        copied: Int = get_copied_int_from_builtin_val(
            integer=integer, variable_name_suffix=suffix
        )
        return copied

    @final
    def _get_copied_number_from_builtin_val(
        self, *, float_or_num: Union[float, Number], attr_identifier: str
    ) -> Number:
        """
        Get a copied number value from a Python built-in float
        and set an attribute's variable name suffix.

        Parameters
        ----------
        float_or_num : Union[float, Number]
            Target float (or Number) value.
        attr_identifier : str
            Attribute identifier string (e.g., `fill_alpha`).

        Returns
        -------
        copied : Number
            Copied Number value.
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_number_from_builtin_val,
        )

        suffix: str = _get_variable_name_suffix(
            instance=self, attr_identifier=attr_identifier
        )
        copied: Number = get_copied_number_from_builtin_val(
            float_or_num=float_or_num, variable_name_suffix=suffix
        )
        return copied

    @final
    def _get_copied_string_from_builtin_val(
        self, *, string: Union[str, String], attr_identifier: str
    ) -> String:
        """
        Get a copied String value from a Python built-in str
        and set an attribute's variable name suffix.

        Parameters
        ----------
        string : Union[str, String]
            Target string value.
        attr_identifier : str
            Attribute identifier string (e.g., `fill_color`).

        Returns
        -------
        copied : String
            Copied String value.
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_string_from_builtin_val,
        )

        suffix: str = _get_variable_name_suffix(
            instance=self, attr_identifier=attr_identifier
        )
        copied: String = get_copied_string_from_builtin_val(
            string=string, variable_name_suffix=suffix
        )
        return copied

    @final
    def _get_copied_boolean_from_builtin_val(
        self, *, bool_val: Union[bool, Boolean], attr_identifier: str
    ) -> Boolean:
        """
        Get a copied Boolean value from a Python built-in bool
        and set an attribute's variable name suffix.

        Parameters
        ----------
        bool_val : Union[bool, Boolean]
            Target bool value.
        attr_identifier : str
            Attribute identifier string (e.g., `fill_color`).

        Returns
        -------
        copied : Boolean
            Copied Boolean value.
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_boolean_from_builtin_val,
        )

        suffix: str = _get_variable_name_suffix(
            instance=self, attr_identifier=attr_identifier
        )
        copied: Boolean = get_copied_boolean_from_builtin_val(
            bool_val=bool_val, variable_name_suffix=suffix
        )
        return copied


def _get_variable_name_suffix(
    *, instance: AttrToApyscValFromBuiltinMixIn, attr_identifier: str
) -> str:
    """
    Get a target instance's variable name suffix.

    Parameters
    ----------
    instance : AttrToApyscValFromBuiltinMixIn
        A target instance.
    attr_identifier : str
        Attribute identifier string (e.g., `x`).

    Returns
    -------
    suffix : str
        A target instance's variable name suffix.
        This value becomes a blank string if a specified
        instance type is not the `VariableNameSuffixAttrOrVarMixIn`.
    """
    if isinstance(instance, VariableNameSuffixAttrOrVarMixIn):
        suffix: str = instance._get_attr_or_variable_name_suffix(
            value_identifier=attr_identifier
        )
        return suffix

    return ""



"""The class implementation for the outlined `push_pin` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPushPinOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `push_pin` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M14,4v5c0,1.12,0.37,2.16,1,3H9c0.65-0.86,1-1.9,1-3V4H14 M17,2H7C6.45,2,6,2.45,6,3c0,0.55,0.45,1,1,1c0,0,0,0,0,0l1,0v5 c0,1.66-1.34,3-3,3v2h5.97v7l1,1l1-1v-7H19v-2c0,0,0,0,0,0c-1.66,0-3-1.34-3-3V4l1,0c0,0,0,0,0,0c0.55,0,1-0.45,1-1 C18,2.45,17.55,2,17,2L17,2z"/></g></svg>'  # noqa
